( cd usr/lib ; rm -rf libiec61883.so.0 )
( cd usr/lib ; ln -sf libiec61883.so.0.0.0 libiec61883.so.0 )
( cd usr/lib ; rm -rf libiec61883.so )
( cd usr/lib ; ln -sf libiec61883.so.0.0.0 libiec61883.so )
